From stern@rowland.harvard.edu Thu Aug 11 12:55:51 2005
Date: Thu, 11 Aug 2005 15:50:32 -0400 (EDT)
From: Alan Stern <stern@rowland.harvard.edu>
To: Greg KH <greg@kroah.com>
cc: Patrick Mochel <mochel@digitalimplant.org>
Subject: USB: Support unbinding of the usb_generic driver
Message-ID: <Pine.LNX.4.44L0.0508111544080.6745-100000@iolanthe.rowland.org>


This patch (as556) adds support for unbinding the usb_generic "driver".
That driver only binds to USB devices, as opposed to interfaces, and it
does nothing much besides marking which struct device's go with an
overall USB device plus providing suspend/resume methods.  Now that
users can unbind drivers at will using the sysfs "unbind" attribute, we
need a rational way of dealing with USB devices that are no longer under
full control of the USB stack.  The patch handles this by unconfiguring
the device, thereby removing all the interfaces and their associated
drivers and children.

Signed-off-by: Alan Stern <stern@rowland.harvard.edu>
Signed-off-by: Greg Kroah-Hartman <gregkh@suse.de>

---
 drivers/usb/core/usb.c |   10 ++++++++++
 1 files changed, 10 insertions(+)

--- gregkh-2.6.orig/drivers/usb/core/usb.c	2005-08-12 14:48:30.000000000 -0700
+++ gregkh-2.6/drivers/usb/core/usb.c	2005-08-12 14:48:31.000000000 -0700
@@ -65,6 +65,16 @@
 }
 static int generic_remove (struct device *dev)
 {
+	struct usb_device *udev = to_usb_device(dev);
+
+	/* if this is only an unbind, not a physical disconnect, then
+	 * unconfigure the device */
+	if (udev->state == USB_STATE_CONFIGURED)
+		usb_set_configuration(udev, 0);
+
+	/* in case the call failed or the device was suspended */
+	if (udev->state >= USB_STATE_CONFIGURED)
+		usb_disable_device(udev, 0);
 	return 0;
 }
 
